/*
 * Astfin Team  - Zarlink Echo Cancellation Module
 *
 *
 * Copyright (C) 2008, Astfin / uCpbx Ltd.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

/* See #ifndef EC_ZL38065_H bellow. It is to enable definitions for
 * ZL60535 anyway.
 */

#ifndef EC_ZL50235_H
#define EC_ZL50235_H

#ifndef EC_ZL38065_H
// A12,A11 determine the page (are these A13,A12 for BF537 ?) !
#define EC_PAGE_0	0x0000

// ECA & ECB offsets
#define ECA_OFFSET	0x00
#define ECB_OFFSET	0x20

// EC A/B group offset & Number of groups
#define EC_GROUP_OFFSET	0x40

#endif

#define EC16_NUMOFGROUPS	8
#define EC8_NUMOFGROUPS		4
#define EC4_NUMOFGROUPS		2

// Define ZL50235 one EC registers offsets
typedef enum {
#ifndef EC_ZL38065_H
	//--- Page 0
	EC_CONTROL_REG_1					= (0x00+EC_PAGE_0),
	EC_CONTROL_REG_2,					// 0x01
	EC_STATUS_REG,						// 0x02
	EC_FLAT_DELAY_REG					= (0x04+EC_PAGE_0),
	EC_DECAY_STEP_SIZE_REG				= (0x06+EC_PAGE_0),
	EC_DECAY_STEP_NUMBER_REG,			// 0x07
	EC_CONTROL_REG_3,					// 0x08
	EC_CONTROL_REG_4,					// 0x09
#endif
	EC16_NOISE_SCALING,					// 0x0A*
	EC16_NOISE_CONTROL,					// 0x0B*
#ifndef EC_ZL38065_H
	EC_RIN_PEAK_DETECT_REG_L			= (0x0C+EC_PAGE_0),
	EC_RIN_PEAK_DETECT_REG_H,			// 0x0D
	EC_SIN_PEAK_DETECT_REG_L,			// 0x0E
	EC_SIN_PEAK_DETECT_REG_H,			// 0x0F
	EC_ERROR_PEAK_DETECT_REG_L,			// 0x10
	EC_ERROR_PEAK_DETECT_REG_H,			// 0x11
	//EC32_PATH_CHANGE_TIMER,			// 0x12
	//EC32_PATH_CHANGE_SENSIBILITY,		// 0x13
	EC_DTDT_ERL_L						= (0x14+EC_PAGE_0),
	EC_DTDT_ERL_H,						// 0x15
	//EC32_ERLLOW_L,						// 0x16
	//EC32_ERLLOW_H,						// 0x17
	EC_NLP_TRESHOLD_L,					// 0x18
	EC_NLP_TRESHOLD_H,					// 0x19
	EC_STEP_SIZE_MU_L,					// 0x1A
	EC_STEP_SIZE_MU_H,					// 0x1B
	EC_GAIN_PAD_CONTROL_L,				// 0x1C
	EC_GAIN_PAD_CONTROL_H,				// 0x1D
	//EC32_NLP_TRESHOLD_2,				// 0x1E
	//EC32_RIN_LOW_POWER_TRESHOLD,		// 0x1F
#endif
} EC16REGADDR;

#ifndef EC_ZL38065_H
//*** Registers bits definition
// ECA & ECB Control Register 1 bits
#define EC_Reset		(1<<7)	// HI = exec.power-up init of this EC
#define EC_INJDis		(1<<6)	// HI = disable noise injection 
#define EC_BBM			(1<<5)	// HI = Back-to-BAck config.enabled
#define EC_PAD			(1<<4)	// HI = 12dB attenuation into Rin & Rout paths
								// LO = Gains register controls attenuation
#define EC_Bypass		(1<<3)	// HI = in bypassed to out, AFCoeff.s=0, Adapt. stopped
#define EC_AdpDis		(1<<2)	// HI = Stop coeff.s adaptation
// ECA CR1 specific bits
#define ECA_CR1_BIT1	0
#define EC_ExtDl		(1<<0)	// HI = ECA & ECB are cascaded into one 128ms EC
// ECB CR1 specific bits
#define ECB_CR1_BIT1	(1<<1)
#define ECB_CR1_BIT0	0

// ECA & ECB Control Register 2 bits
#define EC_TDis			(1<<7)	// HI = Tone detection disabled
#define EC_PHDis		(1<<6)	// HI = detect 2100Hz no matter of Phase Reversals
#define EC_NLPDis		(1<<5)	// HI = NonLinearProcessor disabled
#define EC_AutoTD		(1<<4)	// HI = EC puts itself in bypass mode if 2100Hz detected
#define EC_NBDis		(1<<3)	// HI = narrow-band detector disabled
#define EC_HPFDis		(1<<2)	// HI = offset nulling HPFs are bypassed in Rin & Sin
#define EC_MuteS		(1<<1)	// HI = data on Sout is muted to quiet code (see doc)
#define EC_MuteR		(1<<0)	// HI = data on Rout is muted to quiet code (see doc)

// ECA & ECB Status Register bits
#define EC_TD			(1<<6)	// HI indicates presence of 2100Hz
#define EC_DTDet		(1<<5)	// HI indicates presence of double-talk condition
//#define EC_ACTIVE		(1<<2)	// HI indicates level of Rin has exceeded the LP treshold
#define EC_TDG			(1<<1)	// HI indicates AutoTD=HI & 2100Hz detected
#define EC_NB			(1<<0)	// HI indicates presence of narrow-band signal on Rin

// ECA & ECB Flat Delay Register bits
// 8 bit value of Flat Delay. Search in datasheet for 'Flat Delay' and/or pgs 26,27.

// ECA & ECB Decay Step Number Register bits
// 8 bit value of Decay Step Number (NS). See datasheet pages 26,27.

// ECA & ECB Decay Step Size Control Register bits
// 3 bit value of Decay Step Size (SS). See datasheet pages 26,27.

// ECA & ECB Control Register 3 bits
#define EC_NLRun2		(1<<7)	// ...Not described in reg.table... Search 'NLRun2' in d.s.
#define EC_InjCtrl		(1<<6)	// ...Not described in reg.table... Search 'InjCtrl' in d.s.
#define EC_NLRun1		(1<<5)	// See datasheet
#define EC_RingClr		(1<<4)	// HI = instability detector is activated.
#define EC_PathClr		(1<<2)	// HI = the current EC estimate will be cleared. See d.s.for more.
#define EC_PathDet		(1<<1)	// HI = path change detector is activated
//#define EC_NMatcj		(1<<0)	// ...Not described in datasheet...
#define EC_NLPSel		(1<<0)	// HI = the Advanced NLP is selected.

// ECA & ECB Control Register 4 bits & fields
#define EC_CR4_BIT7		0
#define EC_CR4_SD2		(1<<6)	// Bits 6,5,4. See datasheet for explanation
#define EC_CR4_SD1		(1<<5)
#define EC_CR4_SD0		(1<<4)
#define EC_SupDec_0		(0<<4)	// And field options
#define EC_SupDec_1		(1<<4)
#define EC_SupDec_2		(2<<4)
#define EC_SupDec_3		(3<<4)
#define EC_SupDec_4		(4<<4)
#define EC_SupDec_5		(5<<4)
#define EC_SupDec_6		(6<<4)
#define EC_SupDec_7		(7<<4)
#define EC_CR4_BIT3		0
#define EC_CR4_Slow2	(1<<2)	// Bits 2,1,0. See datasheet for explanation
#define EC_CR4_Slow1	(1<<1)
#define EC_CR4_Slow0	(1<<0)
#define EC_Slow_0		(0<<0)	// And field options
#define EC_Slow_1		(1<<0)
#define EC_Slow_2		(2<<0)
#define EC_Slow_3		(3<<0)
#define EC_Slow_4		(4<<0)
#define EC_Slow_5		(5<<0)
#define EC_Slow_6		(6<<0)
#define EC_Slow_7		(7<<0)

#endif

//*** See what about NoiseScaling and NoiseControl regs
// ECA & ECB Noise Scaling (EC16_NOISE_SCALING)
// 8 bit value. This register is used to scale the comfort noise up or down.
// Seems similar to ZL38065 Noise Level Scaling Factor Reg which is 16 bit however...

// ECA & ECB Noise Control Register (EC16_NOISE_CONTROL)
// 8 bit register with 4 MSBits reserved.
// Seems similar to ZL38065 Noise Level Control Reg which is 16 bit however...
#define EC_NLInc3		(1<<3)	// Bits 3,2,1,0.
#define EC_NLInc2		(1<<2)
#define EC_NLInc1		(1<<1)
#define EC_NLInc0		(1<<0)
#define EC_NLInc_0		0		// And values
#define EC_NLInc_1		1
#define EC_NLInc_2		2
#define EC_NLInc_3		3
#define EC_NLInc_4		4
#define EC_NLInc_5		5
#define EC_NLInc_6		6
#define EC_NLInc_7		7
#define EC_NLInc_8		8
#define EC_NLInc_9		9
#define EC_NLInc_10		10
#define EC_NLInc_11		11
#define EC_NLInc_12		12
#define EC_NLInc_13		13
#define EC_NLInc_14		14
#define EC_NLInc_15		15

#ifndef EC_ZL38065_H
// ECA & ECB Rin Peak Detect Registers (Hi & Lo) bits (EC_RIN_PEAK_DETECT_REG)
// This 16 bit value allows the user to monitor the Rin peak signal level

// ECA & ECB Sin Peak Detect Registers (Hi & Lo) bits (EC_SIN_PEAK_DETECT_REG)
// This 16 bit value allows the user to monitor the Sin peak signal level

// ECA & ECB Error Peak Detect Registers (Hi & Lo) bits (EC_ERROR_PEAK_DETECT_REG)
// This 16 bit value allows the user to monitor the error signal peak level

// ECA & ECB Double-Talk Detection Registers (Hi & Lo) (EC_DTDT_ERL)
// 16 bit value. See datasheet for more.

// ECA & ECB Non-Linear Processor Threshold Register (Hi & Lo) (EC_NLP_TRESHOLD)
// 16 bit value. See datasheet for more.

// ECA & ECB Adaptation Step Size (Hi & Lo) (EC_STEP_SIZE_MU)
// 16 bit value. See datasheet for more.

// ECA & ECB Gains Register 2 (EC_GAIN_PAD_CONTROL_H )
#define EC_GR2_BIT7		0
#define EC_Rin2			(1<<6)	// Bits 6,5,4.
#define EC_Rin1			(1<<5)
#define EC_Rin0			(1<<4)
#define EC_Rin_m12dB	(0<<4)	// And field options
#define EC_Rin_m9dB		(1<<4)
#define EC_Rin_m6dB		(2<<4)
#define EC_Rin_m3dB		(3<<4)
#define EC_Rin_0dB		(4<<4)
#define EC_Rin_p3dB		(5<<4)
#define EC_Rin_p6dB		(6<<4)
#define EC_Rin_p9dB		(7<<4)
#define EC_Rout2		(1<<2)	// Bits 2,1,0.
#define EC_Rout1		(1<<1)
#define EC_Rout0		(1<<0)
#define EC_Rout_m12dB	(0<<0)	// And field options
#define EC_Rout_m9dB	(1<<0)
#define EC_Rout_m6dB	(2<<0)
#define EC_Rout_m3dB	(3<<0)
#define EC_Rout_0dB		(4<<0)
#define EC_Rout_p3dB	(5<<0)
#define EC_Rout_p6dB	(6<<0)
#define EC_Rout_p9dB	(7<<0)

// ECA & ECB Gains Register 1 (EC_GAIN_PAD_CONTROL_L )
#define EC_GR1_BIT7		0
#define EC_Sin2			(1<<6)	// Bits 6,5,4.
#define EC_Sin1			(1<<5)
#define EC_Sin0			(1<<4)
#define EC_Sin_m12dB	(0<<4)	// And field options
#define EC_Sin_m9dB		(1<<4)
#define EC_Sin_m6dB		(2<<4)
#define EC_Sin_m3dB		(3<<4)
#define EC_Sin_0dB		(4<<4)
#define EC_Sin_p3dB		(5<<4)
#define EC_Sin_p6dB		(6<<4)
#define EC_Sin_p9dB		(7<<4)
#define EC_Sout2		(1<<2)	// Bits 2,1,0.
#define EC_Sout1		(1<<1)
#define EC_Sout0		(1<<0)
#define EC_Sout_m12dB	(0<<0)	// And field options
#define EC_Sout_m9dB	(1<<0)
#define EC_Sout_m6dB	(2<<0)
#define EC_Sout_m3dB	(3<<0)
#define EC_Sout_0dB		(4<<0)
#define EC_Sout_p3dB	(5<<0)
#define EC_Sout_p6dB	(6<<0)
#define EC_Sout_p9dB	(7<<0)

//*** Main & common registers
typedef enum {
	EC_MAIN_CTRL_REG_0	= 0x400,
	EC_MAIN_CTRL_REG_1,
	EC_MAIN_CTRL_REG_2,
	EC_MAIN_CTRL_REG_3,
	EC_MAIN_CTRL_REG_4,
	EC_MAIN_CTRL_REG_5,
	EC_MAIN_CTRL_REG_6,
	EC_MAIN_CTRL_REG_7,
	EC_INTR_FIFO_REG	= 0x410,
//	EC_TEST_REG
} MAIN_CMMN_CTRL_REGS_ADDR;

// EC_MAIN_CTRL_REG_0 bits
#define EC_WR_all	(1<<7)	// HI = Useful to initialize the 16 Groups of 
							// Echo Cancellers as per Group 0.
#define EC_ODE		(1<<6)	// When both ODE bit and ODE input pin are high,
							// the Rout and Sout outputs are enabled.
#define EC_MIRQ		(1<<5)	// When low, the Tone Detectors Interrupts are active.
// EC_MAIN_CTRL_REG_0..15 bits
#define EC_MTDBI	(1<<4)	// When low, the Tone Detector B Interrupt is active.
#define EC_MTDAI	(1<<3)	// When low, the Tone Detector A Interrupt is active.
#define EC_Format	(1<<2)	// HI = ECA/ECB accept ITU-T (G.711) PCM code
#define EC_Law		(1<<1)	// HI = ECA/ECB accept A-Law
#define EC_PWUP		(1<<0)	// Power-Up. See datasheet

// EC_INTR_FIFO_REG bits
#define EC_IRQ			(1<<7)
#define EC_IRQ_BIT6		0
#define EC_IRQ_BIT5		0
#define EC_INTR_MASK	0x1F

#endif

#endif
